@@ -1,5 +1,5 @@
 '''
-Copyright (c) 2012, Agora Games, LLC All rights reserved.
+Copyright (c) 2012-2013, Agora Games, LLC All rights reserved.
 
 https://github.com/agoragames/kairos/blob/master/LICENSE.txt
 '''
@@ -8,12 +8,42 @@ from exceptions import *
 import operator
 import sys
 import time
+import re
 
 if sys.version_info[:2] > (2, 6):
     from collections import OrderedDict
 else:
     from ordereddict import OrderedDict
 
+NUMBER_TIME = re.compile('^[\d]+$')
+SIMPLE_TIME = re.compile('^([\d]+)([hdwmy])$')
+
+SIMPLE_TIMES = {
+  'h' : 60*60,        # hour
+  'd' : 60*60*24,     # day
+  'w' : 60*60*24*7,   # week
+  'm' : 60*60*24*30,  # month(-ish)
+  'y' : 60*60*24*365, # year(-ish)
+}
+
+def _resolve_time(value):
+  '''
+  Resolve the time in seconds of a configuration value.
+  '''
+  if value is None or isinstance(value,(int,long)):
+    return value
+
+  if NUMBER_TIME.match(value):
+    return long(value)
+
+  simple = SIMPLE_TIME.match(value)
+  if SIMPLE_TIME.match(value):
+    multiplier = long( simple.groups()[0] )
+    constant = SIMPLE_TIMES[ simple.groups()[1] ]
+    return multiplier * constant
+
+  raise ValueError('Unsupported time format %s'%value)
+
 class Timeseries(object):
   '''
   Base class of all time series. Also acts as a factory to return the correct
@@ -102,9 +132,9 @@ class Timeseries(object):
 
     # Preprocess the intervals
     for interval,config in self._intervals.iteritems():
-      step = int( config.get('step') )  # Required
-      steps = config.get('steps',None)  # Optional
-      resolution = int( config.get('resolution',step) ) # Optional
+      step = _resolve_time( config['step'] ) # Required
+      steps = config.get('steps',None)       # Optional
+      resolution = _resolve_time( config.get('resolution',step) ) # Optional
 
       def calc_keys(name, timestamp, s=step, r=resolution, i=interval):
         interval_bucket = int( timestamp/s )
@@ -120,7 +150,7 @@ class Timeseries(object):
       config['calc_keys'] = calc_keys
       config['expire'] = expire
       config['coarse'] = (resolution==step)
-        
+
   def insert(self, name, value, timestamp=None):
     '''
     Insert a value for the timeseries "name". For each interval in the 
@@ -228,7 +258,7 @@ class Timeseries(object):
       rval = { i_bucket*config['step'] : self._condense(rval) }
     return rval
   
-  def series(self, name, interval, steps=None, condensed=False):
+  def series(self, name, interval, steps=None, condensed=False, timestamp=None):
     '''
     Return all the data in a named time series for a given interval. If steps
     not defined and there are none in the config, defaults to 1.
@@ -240,6 +270,8 @@ class Timeseries(object):
     '''
     # TODO: support start and end timestamps
     # TODO: support other ways of declaring the interval
+    if not timestamp:
+      timestamp = time.time()
 
     config = self._intervals.get(interval)
     if not config:
@@ -248,7 +280,7 @@ class Timeseries(object):
     steps = steps if steps else config.get('steps',1)
     resolution = config.get('resolution',step)
 
-    end_timestamp = time.time()
+    end_timestamp = timestamp
     end_bucket = int( end_timestamp / step )
     start_bucket = end_bucket - steps +1 # +1 because it's inclusive of end
 
@@ -388,13 +420,21 @@ class Count(Timeseries):
   '''
   Time series that simply increments within each interval.
   '''
+  
+  def insert(self, name, value=1, timestamp=None):
+    super(Count,self).insert(name, value, timestamp)
 
   # TODO: Let the count timeseries support positive and negative 
   def _insert(self, handle, key, value):
     '''
     Insert the value into the series.
     '''
-    handle.incr(key)
+    if value==1:
+      handle.incr(key)
+    elif isinstance(value,float):
+      handle.incrbyfloat(key, value)
+    elif value!=0:
+      handle.incrby(key,value)
   
   def _get(self, handle, key):
     return handle.get(key)
